/******************************************************************************
 *                                                                            *
 * This work is licensed under the Creative Commons                           *
 * Attribution-NoDerivs-NonCommercial License. To view a copy of this         *
 * license, visit http://creativecommons.org/licenses/by-nd-nc/1.0/ or send a *
 * letter to Creative Commons, 559 Nathan Abbott Way, Stanford, California    *
 * 94305, USA.                                                                *
 *                                                                            *
 *                                                                            *
 *                               (cc) creative                                *
 *                                     commons                                *
 *                                COMMONS DEED                                *
 *                                                                            *
 *                  Attribution-NonCommercial-ShareAlike 1.0                  *
 *                                                                            *
 * You are free:                                                              *
 *   -to copy, distribute, display, and perform the work                     *
 *   - to make derivative works                                               *
 *                                                                            *
 * Under the following conditions:                                            *
 *   BY: Attribution. You must give the original author credit.               *
 *   NC: Noncommercial. You may not use this work for commercial purposes.    *
 *   SA: Share Alike. If you alter, transform, or build upon this work, you   *
 *       may distribute the resulting work only under a license identical to  *
 *       this one.                                                            *
 *                                                                            *
 *   -For any reuse or distribution, you must make clear to others the       *
 *     license terms of this work.                                            *
 *   -Any of these conditions can be waived if you get permission from the   *
 *     author.                                                                *
 *                                                                            *
 * Your fair use and other rights are in no way affected by the above.        *
 *                                                                            *
 * This is a human-readable summary of the Legal Code (the full license)      *
 * available on http://creativecommons.org/licenses/by-nc-sa/1.0/legalcode    *
 *                                                                            *
 *                                                                            *
 * Disclaimer:                                                                *
 *                                                                            *
 *   The Commons Deed is not a license. It is simply a handy reference for    *
 *   understanding the Legal Code (the full license) - it is a human-readable *
 *   expression of some of its key terms. Think of it as the user-friendly    *
 *   interface to the Legal Code beneath. This Deed itself has no legal       *
 *   value, and its contents do not appear in the actual license.             *
 *                                                                            *
 *   Creative Commons is not a law firm and does not provide legal services.  *
 *   Distributing of, displaying of, or linking to this Commons Deed does not *
 *   create an attorney-client relationship.                                  *
 *                                                                            *
 ******************************************************************************/
package net.jourand.placide.io;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.ObjectOutputStream;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Vector;

import net.jourand.placide.places.Arrondissement;
import net.jourand.placide.places.Canton;
import net.jourand.placide.places.Commune;
import net.jourand.placide.places.Departement;
import net.jourand.placide.places.HistoricalCommune;
import net.jourand.placide.places.Place;
import net.jourand.placide.places.Places;
import net.jourand.placide.places.Region;
import net.jourand.placide.tools.Sorter;


public final class DataBuilder {
    private static final ClassLoader cl = DataBuilder.class.getClassLoader();
    private Region[] regions = null;
    private Departement[] departements = null;
    private Arrondissement[] arrondissements = null;
    private Canton[] cantons = null;
    private Commune[] communes = null;
    private HistoricalCommune[] histories = null;
    private Places places = null;

    private void build(String root) {
        Commune[] searchables = null;

        places = new Places();

        System.out.println("Building regions...");
        regions = buildRegions("resources/region.txt");
        places.setRegions(regions);
        System.out.println("Regions built");

        System.out.println("Building departements...");
        departements = buildDepartements("resources/depts.txt");
        places.setDepartements(departements);
        System.out.println("Departements built");

        System.out.println("Building arrondissements...");
        arrondissements = buildArrondissements("resources/arrond.txt");
        places.setArrondissements(arrondissements);
        System.out.println("Arrondissements built");

        System.out.println("Building cantons...");
        cantons = buildCantons("resources/canton.txt");
        places.setCantons(cantons);
        System.out.println("Cantons built");

        System.out.println("Building communes...");
        communes = buildCommunes("resources/france.txt", "resources/zip.txt");
        Sorter.sortByName(communes);
        System.out.println("Communes built");

        System.out.println("Updating regions...");
        updateRegions("resources/region.txt");
        System.out.println("Regions updated");

        System.out.println("Updating regions...");
        updateDepartements("resources/depts.txt");
        System.out.println("Regions updated");

        System.out.println("Updating arrondissements...");
        updateArrondissements("resources/arrond.txt");
        System.out.println("Arrondissements updated");

        System.out.println("Updating cantons...");
        updateCantons("resources/canton.txt");
        System.out.println("Cantons updated");

        System.out.println("Building communes history...");
        histories = buildHistoricalCommune("resources/historiq.txt");
        System.out.println("Communes history built");

        System.out.println("Retrieving searchable communes...");
        searchables = retrieveSearchables();
        Sorter.sortByName(searchables);
        places.setSearchables(searchables);
        System.out.println("Searchable communes retrieved");

        places.setCommunes(communes);
        places.setHistories(histories);

        System.out.println("Serializing places...");
        serialize(places, root + "data/places.ser");
        System.out.println("Places serialized...");
    }

    private Region[] buildRegions(String resourceName) {
        String line = null;
        StringTokenizer st = null;
        String code = null;
        short nameType = 0;
        String name = null;
        Region region = null;
        Vector rVector = null;
        BufferedReader br = null;
        InputStreamReader isr = null;

        rVector = new Vector();
        isr = new InputStreamReader(cl.getResourceAsStream(resourceName));
        br = new BufferedReader(isr);

        try {
            line = br.readLine();
            line = br.readLine();

            while (line != null) {
                st = new StringTokenizer(line, "\t");
                code = st.nextToken();
                st.nextToken();
                nameType = Short.parseShort(st.nextToken());
                st.nextToken();
                name = st.nextToken();
                region = new Region();
                region.setCode(code);
                region.setNameType(nameType);
                region.setName(name);
                rVector.addElement(region);
                line = br.readLine();
            }

            region.setName(name);
            regions = (Region[]) rVector.toArray((Object[]) new Region[1]);
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } finally {
            try {
                br.close();
            } catch (IOException ioe) {
                ioe.printStackTrace();
            }
        }

        return regions;
    }

    private void updateRegions(String resourceName) {
        String line = null;
        StringTokenizer st = null;
        String code = null;
        String mainCode = null;
        Region region = null;
        Commune commune = null;
        BufferedReader br = null;
        InputStreamReader isr = null;

        isr = new InputStreamReader(cl.getResourceAsStream(resourceName));
        br = new BufferedReader(isr);

        try {
            line = br.readLine();
            line = br.readLine();

            while (line != null) {
                st = new StringTokenizer(line, "\t");
                code = st.nextToken();
                mainCode = st.nextToken();
                region = findRegion(code);
                commune = findCommune(mainCode);
                region.setMainPlace(commune);
                line = br.readLine();
            }
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } finally {
            try {
                br.close();
            } catch (IOException ioe) {
                ioe.printStackTrace();
            }
        }
    }

    private Departement[] buildDepartements(String resourceName) {
        String line = null;
        StringTokenizer st = null;
        String code = null;
        short nameType = 0;
        String regionCode = null;
        String name = null;
        Departement[] departements = null;
        Departement departement = null;
        Vector dVector = null;
        BufferedReader br = null;
        InputStreamReader isr = null;

        dVector = new Vector();
        isr = new InputStreamReader(cl.getResourceAsStream(resourceName));
        br = new BufferedReader(isr);

        try {
            line = br.readLine();
            line = br.readLine();

            while (line != null) {
                st = new StringTokenizer(line, "\t");
                regionCode = st.nextToken();
                code = st.nextToken();
                st.nextToken();
                nameType = Short.parseShort(st.nextToken());
                st.nextToken();
                name = st.nextToken();
                departement = new Departement();
                departement.setCode(code);
                departement.setNameType(nameType);
                departement.setName(name);
                departement.setParent(findRegion(regionCode));
                dVector.addElement(departement);
                line = br.readLine();
            }

            departements = (Departement[]) dVector.toArray((Object[]) new Departement[1]);
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } finally {
            try {
                br.close();
            } catch (IOException ioe) {
                ioe.printStackTrace();
            }
        }

        return departements;
    }

    private void updateDepartements(String resourceName) {
        String line = null;
        StringTokenizer st = null;
        String code = null;
        String mainCode = null;
        Departement departement = null;
        Commune commune = null;
        BufferedReader br = null;
        InputStreamReader isr = null;

        isr = new InputStreamReader(cl.getResourceAsStream(resourceName));
        br = new BufferedReader(isr);

        try {
            line = br.readLine();
            line = br.readLine();

            while (line != null) {
                st = new StringTokenizer(line, "\t");
                st.nextToken();
                code = st.nextToken();
                mainCode = st.nextToken();
                departement = findDepartement(code);
                commune = findCommune(mainCode);
                departement.setMainPlace(commune);
                line = br.readLine();
            }
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } finally {
            try {
                br.close();
            } catch (IOException ioe) {
                ioe.printStackTrace();
            }
        }
    }

    private Arrondissement[] buildArrondissements(String resourceName) {
        String line = null;
        String code = null;
        short nameType = 0;
        String departementCode = null;
        String article = null;
        String name = null;
        Arrondissement[] arrondissements = null;
        Arrondissement arrondissement = null;
        Vector aVector = null;
        BufferedReader br = null;
        InputStreamReader isr = null;

        aVector = new Vector();
        isr = new InputStreamReader(cl.getResourceAsStream(resourceName));
        br = new BufferedReader(isr);

        try {
            line = br.readLine();
            line = br.readLine();

            while (line != null) {
                line = line.substring(line.indexOf("\t", 1));
                departementCode = line.substring(1, line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                code = line.substring(1, line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                line.substring(1, line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                nameType = Short.parseShort(line.substring(1,
                                                           line.indexOf("\t", 1)));
                line = line.substring(line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                article = line.substring(1, line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                name = line.substring(1);
                arrondissement = new Arrondissement();
                arrondissement.setCode(code);
                arrondissement.setNameType(nameType);
                arrondissement.setName(name);
                arrondissement.setParent(findDepartement(departementCode));
                aVector.addElement(arrondissement);
                line = br.readLine();
            }

            arrondissements = (Arrondissement[]) aVector.toArray((Object[]) new Arrondissement[1]);
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } finally {
            try {
                br.close();
            } catch (IOException ioe) {
                ioe.printStackTrace();
            }
        }

        return arrondissements;
    }

    private void updateArrondissements(String resourceName) {
        String line = null;
        String departementCode = null;
        String code = null;
        String mainCode = null;
        Arrondissement arrondissement = null;
        Commune commune = null;
        BufferedReader br = null;
        InputStreamReader isr = null;

        isr = new InputStreamReader(cl.getResourceAsStream(resourceName));
        br = new BufferedReader(isr);

        try {
            line = br.readLine();
            line = br.readLine();

            while (line != null) {
                line = line.substring(line.indexOf("\t", 1));
                departementCode = line.substring(1, line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                code = line.substring(1, line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                mainCode = line.substring(1, line.indexOf("\t", 1));
                arrondissement = findArrondissement(departementCode, code);
                commune = findCommune(mainCode);
                arrondissement.setMainPlace(commune);
                line = br.readLine();
            }
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } finally {
            try {
                br.close();
            } catch (IOException ioe) {
                ioe.printStackTrace();
            }
        }
    }

    private Canton[] buildCantons(String resourceName) {
        String line = null;
        String article = null;
        String str = null;
        short type = 0;
        String code = null;
        short nameType = 0;
        String name = null;
        Canton[] cantons = null;
        Canton canton = null;
        Vector cVector = null;
        BufferedReader br = null;
        InputStreamReader isr = null;
        String departementCode = null;
        String arrondissementCode = null;

        cVector = new Vector();
        isr = new InputStreamReader(cl.getResourceAsStream(resourceName));
        br = new BufferedReader(isr);

        try {
            line = br.readLine();
            line = br.readLine();

            while (line != null) {
                line = line.substring(line.indexOf("\t", 1));
                departementCode = line.substring(1, line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                arrondissementCode = line.substring(1, line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                code = line.substring(1, line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                type = Short.parseShort(line.substring(1, line.indexOf("\t", 1)));
                line = line.substring(line.indexOf("\t", 1));
                line.substring(1, line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                str = line.substring(1, line.indexOf("\t", 1));

                if ((str != null) && (!"".equals(str))) {
                    nameType = Short.parseShort(str);
                }

                line = line.substring(line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                article = line.substring(1, line.indexOf("\t", 1));

                if ((article != null) && (!"".equals(article))) {
                    article = article.substring(1, article.length() - 1);
                } else {
                    article = null;
                }

                line = line.substring(line.indexOf("\t", 1));
                name = line.substring(1);
                canton = new Canton();
                canton.setCode(code);
                canton.setCantonType(type);
                canton.setNameType(nameType);
                canton.setName(name);
                canton.setParent(findArrondissement(departementCode,
                                                    arrondissementCode));
                cVector.addElement(canton);
                line = br.readLine();
            }

            cantons = (Canton[]) cVector.toArray((Object[]) new Canton[1]);
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } finally {
            try {
                br.close();
            } catch (IOException ioe) {
                ioe.printStackTrace();
            }
        }

        return cantons;
    }

    private void updateCantons(String resourceName) {
        String line = null;
        String code = null;
        String mainCode = null;
        Canton canton = null;
        Commune commune = null;
        BufferedReader br = null;
        InputStreamReader isr = null;
        String departementCode = null;

        isr = new InputStreamReader(cl.getResourceAsStream(resourceName));
        br = new BufferedReader(isr);

        try {
            line = br.readLine();
            line = br.readLine();

            while (line != null) {
                line = line.substring(line.indexOf("\t", 1));
                departementCode = line.substring(1, line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                code = line.substring(1, line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                mainCode = line.substring(1, line.indexOf("\t", 1));
                canton = findCanton(departementCode, code);
                commune = findCommune(mainCode);
                canton.setMainPlace(commune);
                line = br.readLine();
            }
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } finally {
            try {
                br.close();
            } catch (IOException ioe) {
                ioe.printStackTrace();
            }
        }
    }

    private Commune[] buildCommunes(String inseeResourceName,
                                    String zipResourceName) {
        String line = null;
        String str = null;
        short actual = 0;
        String departementCode = null;
        String code = null;
        String arrondissementCode = null;
        String cantonCode = null;
        boolean modified = false;
        String poleCode = null;
        short nameType = 0;
        String article = null;
        String name = null;
        Commune commune = null;
        Vector cVector = null;
        BufferedReader br = null;
        InputStreamReader isr = null;
        Hashtable poles = null;
        Enumeration poleCodes = null;

        cVector = new Vector();
        poles = new Hashtable();
        isr = new InputStreamReader(cl.getResourceAsStream(inseeResourceName));
        br = new BufferedReader(isr);

        try {
            line = br.readLine();
            line = br.readLine();

            while (line != null) {
                actual = Short.parseShort(line.substring(0,
                                                         line.indexOf("\t", 1)));
                line = line.substring(line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                str = line.substring(1, line.indexOf("\t", 1));
                departementCode = line.substring(1, line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                code = departementCode
                       + line.substring(1, line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                arrondissementCode = line.substring(1, line.indexOf("\t", 1));

                if ((arrondissementCode != null)
                    && (!"".equals(arrondissementCode))) {
                    arrondissementCode = null;
                }

                line = line.substring(line.indexOf("\t", 1));
                cantonCode = line.substring(1, line.indexOf("\t", 1));

                if ((cantonCode != null) && ("".equals(cantonCode))) {
                    cantonCode = null;
                }

                line = line.substring(line.indexOf("\t", 1));

                if ("0".equals(line.substring(1, line.indexOf("\t", 1)))) {
                    modified = false;
                } else {
                    modified = true;
                }

                line = line.substring(line.indexOf("\t", 1));
                str = line.substring(1, line.indexOf("\t", 1));

                if ((str != null) && (!"".equals(str))) {
                    poleCode = str;
                } else {
                    poleCode = null;
                }

                line = line.substring(line.indexOf("\t", 1));
                nameType = Short.parseShort(line.substring(1,
                                                           line.indexOf("\t", 1)));
                line = line.substring(line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                article = line.substring(1, line.indexOf("\t", 1));

                if ((article != null) && (!"".equals(article))) {
                    article = article.substring(1, article.length() - 1);
                } else {
                    article = null;
                }

                line = line.substring(line.indexOf("\t", 1));
                name = line.substring(1, line.indexOf("\t", 1));
                commune = new Commune();
                commune.setActual(actual);
                commune.setCode(code);
                commune.setNameType(nameType);
                commune.setName(name);

                if (cantonCode != null) {
                    commune.setParent(findCanton(departementCode, cantonCode));
                }

                if (commune.getParent() == null) {
                    commune.setParent(findDepartement(departementCode));
                }

                if (commune.getParent() == null) {
                    System.out.println("No parent for commune "
                                       + commune.getCode());
                }

                if (poleCode != null) {
                    poles.put(commune, poleCode);
                }

                cVector.addElement(commune);
                line = br.readLine();
            }

            communes = (Commune[]) cVector.toArray((Object[]) new Commune[1]);
            poleCodes = poles.keys();

            while (poleCodes.hasMoreElements()) {
                commune = (Commune) poleCodes.nextElement();
                poleCode = (String) poles.get(commune);
                commune.setPole(findCommune(poleCode));
            }

            setPostalCodes(zipResourceName);
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } finally {
            try {
                br.close();
            } catch (IOException ioe) {
                ioe.printStackTrace();
            }
        }

        return communes;
    }

    private void setPostalCodes(String zipResourceName) {
        BufferedReader br = null;
        InputStreamReader isr = null;
        String inseeCode = null;
        String prevCode = null;
        String postalCode = null;
        Commune commune = null;
        String line = null;

        isr = new InputStreamReader(cl.getResourceAsStream(zipResourceName));
        br = new BufferedReader(isr);

        try {
            line = br.readLine();
            line = br.readLine();

            while (line != null) {
                line = line.substring(line.indexOf(";"));
                postalCode = line.substring(1, line.indexOf(";", 1));
                line = line.substring(line.indexOf(";", 1));
                line = line.substring(line.indexOf(";", 1));
                inseeCode = line.substring(1);

                if (inseeCode.length() == 4) {
                    inseeCode = "0" + inseeCode;
                }

                if (!inseeCode.startsWith("20")) {
                    commune = findCommune(inseeCode);

                    if (commune != null) {
                        if ((commune.getActual() == 1)
                            || (commune.getActual() == 2)
                            || (commune.getActual() == 5)
                            || (commune.getActual() == 6)) {
                            commune.setPostalCode(postalCode);
                        }
                    } else {
                    	if(postalCode.equals("69100")) {
                    	System.out.println("No commune found with code "
                    		+ inseeCode + " for postal code "
                    		+ postalCode);
                    	}
                    }
                } else {
                    inseeCode = "2A" + inseeCode.substring(2);
                    commune = findCommune(inseeCode);

                    if (commune != null) {
                        commune.setPostalCode(postalCode);
                    } else {
                        inseeCode = "2B" + inseeCode.substring(2);
                        commune = findCommune(inseeCode);

                        if (commune != null) {
                            if ((commune.getActual() == 1)
                                || (commune.getActual() == 2)) {
                                commune.setPostalCode(postalCode);
                            }
	                    } else {
/*
	                    	System.out.println("No commune found with code "
	                    		+ inseeCode + " for postal code "
	                    		+ postalCode);
*/
                        }
                    }
                }

                if (!inseeCode.substring(0, 2).equals(prevCode)) {
                    if (!inseeCode.substring(0, 2).equals("2A")
                        && !inseeCode.substring(0, 2).equals("2B")) {
                        prevCode = inseeCode.substring(0, 2);
                        System.gc();
                    }
                }

                line = br.readLine();
            }
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } finally {
            try {
                br.close();
            } catch (IOException ioe) {
                ioe.printStackTrace();
            }
        }
    }

    private HistoricalCommune[] buildHistoricalCommune(String historyResourceName) {
        HistoricalCommune history = null;
        String line = null;
        Vector hVector = null;
        SimpleDateFormat sdf = null;
        BufferedReader br = null;
        InputStreamReader isr = null;
        String str = null;
        String departementCode = null;
        String code = null;
        HistoricalCommune[] histories = null;

        hVector = new Vector();
        isr = new InputStreamReader(cl.getResourceAsStream(historyResourceName));
        br = new BufferedReader(isr);

        sdf = new SimpleDateFormat("dd-MM-yyyy");

        try {
            line = br.readLine();
            line = br.readLine();

            while (line != null) {
                history = new HistoricalCommune();
                departementCode = line.substring(0, line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));

                //        history.arrondissementCode = line.substring(1, line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));

                //        history.cantonCode = line.substring(1, line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                code = departementCode
                       + line.substring(1, line.indexOf("\t", 1));
                history.setCode(code);

                //        history.code = history.departementCode+line.substring(1, line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                str = line.substring(1, line.indexOf("\t", 1));

                if ((str != null) && (!"".equals(str))) {
                    str = str.substring(0, 1);

                    if ("D".equals(str)) {
                        history.setLegalType((short) 0);
                    } else if ("O".equals(str)) {
                        history.setLegalType((short) 1);
                    } else if ("L".equals(str)) {
                        history.setLegalType((short) 2);
                    } else if ("A".equals(str)) {
                        history.setLegalType((short) 3);
                    } else if ("C".equals(str)) {
                        history.setLegalType((short) 4);
                    } else if ("R".equals(str)) {
                        history.setLegalType((short) 5);
                    } else if ("I".equals(str)) {
                        history.setLegalType((short) 6);
                    } else {
                        System.out.println("Unknown legal type: " + str);
                    }

                    str = line.substring(2, line.indexOf("\t", 1));

                    if ((str != null) && (!"".equals(str))) {
                        history.setLegalDate(sdf.parse(str, new ParsePosition(0)));
                    }
                }

                line = line.substring(line.indexOf("\t", 1));
                str = line.substring(1, line.indexOf("\t", 1));

                if ((str != null) && (!"".equals(str))) {
                    history.setJoDate(sdf.parse(str, new ParsePosition(0)));
                }

                line = line.substring(line.indexOf("\t", 1));
                str = line.substring(1, line.indexOf("\t", 1));

                if ((str != null) && (!"".equals(str))) {
                    history.setEffectiveDate(sdf.parse(str, new ParsePosition(0)));
                }

                line = line.substring(line.indexOf("\t", 1));
                str = line.substring(1, line.indexOf("\t", 1));

                if ((str != null) && (!"".equals(str))) {
                    history.setMostRecentDate(sdf.parse(str,
                                                        new ParsePosition(0)));
                }

                line = line.substring(line.indexOf("\t", 1));
                history.setModification(Short.parseShort(line.substring(1,
                                                                        line
                                                                        .indexOf("\t",
                                                                                 1))));
                line = line.substring(line.indexOf("\t", 1));
                str = line.substring(1, line.indexOf("\t", 1));

                if ((str != null) && (!"".equals(str))) {
                    history.setChefLieuCode(str);
                }

                line = line.substring(line.indexOf("\t", 1));
                str = line.substring(1, line.indexOf("\t", 1));

                if ((str != null) && (!"".equals(str))) {
                    history.setOldChefLieuCode(str);
                }

                line = line.substring(line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                str = line.substring(1, line.indexOf("\t", 1));

                if ((str != null) && (!"".equals(str))) {
                    history.setFusionnedCode(str);
                }

                line = line.substring(line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                line = line.substring(line.indexOf("\t", 1));
                str = line.substring(1, line.indexOf("\t", 1));

                if ((str != null) && (!"".equals(str))) {
                    history.setOldCode(str);
                }

                line = line.substring(line.indexOf("\t", 1));
                str = line.substring(1, line.indexOf("\t", 1));

                if ((str != null) && (!"".equals(str))) {
                    history.setOldArrondissementCode(str);
                }

                line = line.substring(line.indexOf("\t", 1));
                str = line.substring(1, line.indexOf("\t", 1));

                if ((str != null) && (!"".equals(str))) {
                    history.setOldCantonCode(str);
                }

                line = line.substring(line.indexOf("\t", 1));
                str = line.substring(1, line.indexOf("\t", 1));

                if ((str != null) && (!"".equals(str))) {
                    history.setNewNameType(Short.parseShort(str));
                }

                line = line.substring(line.indexOf("\t", 1));
                history.setNewName(line.substring(1, line.indexOf("\t", 1)));
                line = line.substring(line.indexOf("\t", 1));
                str = line.substring(1, line.indexOf("\t", 1));

                if ((str != null) && (!"".equals(str))) {
                    history.setNameType(Short.parseShort(str));
                }

                line = line.substring(line.indexOf("\t", 1));
                line = line.substring(1);

                if ((line != null) && (!"".equals(line))) {
                    history.setName(line);
                }

                hVector.addElement(history);
                line = br.readLine();
            }

            histories = (HistoricalCommune[]) hVector.toArray((Object[]) new HistoricalCommune[1]);

            for (int i = 0; i < histories.length; i++) {
                histories[i].setParent(findCommune(histories[i].getCode())
                                           .getParent());
            }
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } finally {
            try {
                br.close();
            } catch (IOException ioe) {
                ioe.printStackTrace();
            }
        }

        return histories;
    }

    private Region findRegion(String code) {
        Region region = null;

        for (int i = 0; (i < regions.length) && (region == null); i++) {
            if (regions[i].getCode().equals(code)) {
                region = regions[i];
            }
        }

        return region;
    }

    public Departement findDepartement(String code) {
        Departement departement = null;

        for (int i = 0; (i < departements.length) && (departement == null);
             i++) {
            if (departements[i].getCode().equals(code)) {
                departement = departements[i];
            }
        }

        return departement;
    }

    public Arrondissement findArrondissement(String departementCode, String code) {
        Arrondissement arrondissement = null;

        for (int i = 0;
             (i < arrondissements.length) && (arrondissement == null); i++) {
            if (arrondissements[i].getParent().getCode().equals(departementCode)) {
                if (arrondissements[i].getCode().equals(code)) {
                    arrondissement = arrondissements[i];
                }
            }
        }

        return arrondissement;
    }

    public Canton findCanton(String departementCode, String code) {
        Canton canton = null;

        for (int i = 0; (i < cantons.length) && (canton == null); i++) {
            if (cantons[i].getParent().getParent().getCode().equals(departementCode)) {
                if (cantons[i].getCode().equals(code)) {
                    canton = cantons[i];
                }
            }
        }

        return canton;
    }

    public Commune findCommune(String code) {
        Commune   commune = null;
        Commune[] coms    = null;

		coms = new Commune[7];
        for (int i = 0; i < communes.length; i++) {
            if (communes[i].getCode().equals(code)) {
                coms[communes[i].getActual()] = communes[i];
            }
        }
        
        for(int i=1; (i < 7) && (commune == null); i++) {
        	if(coms[i] != null) {
        		commune = coms[i];
        	}
        }

        return commune;
    }

    private Commune[] retrieveSearchables() {
        Vector searchables = null;
        Commune commune = null;

        searchables = new Vector();

        for (int i = 0; i < communes.length; i++) {
            if (communes[i].getActual() < 4) {
                searchables.addElement(communes[i]);
            }
        }

        for (int i = 0; i < histories.length; i++) {
            commune = findCommune(histories[i].getCode());

            if (histories[i].getModification() < 20) {
                if (!histories[i].getLongName().equals(commune.getLongName())) {
                    searchables.addElement(histories[i]);
                }
            }
        }

        return (Commune[]) searchables.toArray((Object[]) new Commune[1]);
    }

    private void serialize(Object object, String serFile) {
        FileOutputStream fos = null;
        ObjectOutputStream oos = null;

        try {
            fos = new FileOutputStream(serFile);
            oos = new ObjectOutputStream(fos);
            oos.writeObject(object);
            oos.close();
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } finally {
            try {
                oos.close();
            } catch (IOException ioe) {
                ioe.printStackTrace();
            }
        }
    }

    public static void main(String[] args) {
        DataBuilder db = null;
        String root = null;
        long b = 0;
        long e = 0;
        int m = 0;
        int s = 0;

        if (args.length != 1) {
            System.out.println("Usage: DataBuilder rootPath");
        } else {
            root = args[0];

            if (!root.endsWith(File.separator)) {
                root = root + File.separator;
            }

            System.out.println("Data building in progress... BE PATIENT (about 10 minutes on a PIV 2.0 GHz) !!!");
            b = System.currentTimeMillis();
            db = new DataBuilder();
            db.build(root);
            e = System.currentTimeMillis();
            m = (int) ((((float) (e - b)) / ((float) 1000.0)) / 60.0);
            s = (int) ((((float) (e - b)) / ((float) 1000.0)) % 60);
            System.out.println("Built  places (" + db.communes.length
                               + " communes) in " + m + " minutes " + s
                               + " secondes");
        }
    }
}
